package com.ruoyi.api.labour.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.api.base.TokenReqVo;
import com.ruoyi.api.labour.domain.LabourSignetVo;
import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.annotation.RateLimiter;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.enums.HttpStatusEnum;
import com.ruoyi.common.enums.LimitType;
import com.ruoyi.common.enums.ShiFouEnum;
import com.ruoyi.common.enums.UserTypeEnum;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.AuthRsaUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.system.domain.SysApplyConfig;
import com.ruoyi.system.service.ISysUserService;
import com.yanzhu.jh.project.domain.SurProjectAttendanceData;
import com.yanzhu.jh.project.domain.SurProjectAttendanceGroup;
import com.yanzhu.jh.project.domain.SurProjectAttendanceUser;
import com.yanzhu.jh.project.service.ISurProjectAttendanceDataService;
import com.yanzhu.jh.project.service.ISurProjectAttendanceGroupService;
import com.yanzhu.jh.project.service.ISurProjectAttendanceUserService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 劳务人员APIController
 *
 * @author JiangYuQi
 * @date 2024-01-13
 */
@RestController
@RequestMapping("/api/labour")
public class LabourApiController extends BaseController {

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISurProjectAttendanceGroupService surProjectAttendanceGroupService;

    @Autowired
    private ISurProjectAttendanceUserService surProjectAttendanceUserService;

    @Autowired
    private ISurProjectAttendanceDataService surProjectAttendanceDataService;

    /**
     * 获取系统token
     * 限流规则[60秒内最多请求10次，限流策略IP]
     * @param req 请求信息
     * @author JiangYuQi
     * @date 2024-01-13
     */
    @Anonymous
    @ApiOperation(value = "获取TOKEN")
    @RateLimiter(count = 10, limitType = LimitType.IP)
    @PostMapping("/v1/getToken")
    public AjaxResult getToken(@Validated @RequestBody TokenReqVo req) {
        SysApplyConfig sysApplyConfig = redisCache.getCacheObject(CacheConstants.YANZHU_SYSTEM_CONFIG+req.getAppId());
        if(sysApplyConfig==null){
            throw new ServiceException(HttpStatusEnum.ERROR.getInfo(),HttpStatusEnum.ERROR.getCode());
        }
        if(StringUtils.equals(ShiFouEnum.SHI.getCode()+"",sysApplyConfig.getIsDel())){
            throw new ServiceException(HttpStatusEnum.DISABLE.getInfo(),HttpStatusEnum.DISABLE.getCode());
        }
        AjaxResult ajax = AjaxResult.success();
        String systemToken = "";
        if (req.getLoginSign(sysApplyConfig.getPrivateKey())) {
            systemToken = this.getAppIdLoginToken(req.getAppId(),sysApplyConfig);
        } else {
            throw new ServiceException(HttpStatusEnum.SINGET_ERROR.getInfo(),HttpStatusEnum.SINGET_ERROR.getCode());
        }
        ajax.put("systemToken", systemToken);
        return ajax;
    }

    /**
     * appId登录创建用户并模拟登录
     *
     * @param appId 用户账号
     * @return token
     */
    private String getAppIdLoginToken(String appId,SysApplyConfig sysApplyConfig) {
        // 查询用户是否存在，不存在则保存
        SysUser sysUser = userService.selectUserByUserName(appId);
        String password = Convert.toStr(System.currentTimeMillis());
        if (sysUser == null) {
            sysUser = new SysUser();
            sysUser.setDeptId(sysApplyConfig.getDeptId());
            sysUser.setUserName(Convert.toStr(sysApplyConfig.getAppId()));
            sysUser.setNickName(Convert.toStr(sysApplyConfig.getAppId()));
            sysUser.setUserType(UserTypeEnum.APPLY.getCode());
            sysUser.setStatus("0");
            sysUser.setCreateBy("AppId登录创建用户");
            sysUser.setPassword(SecurityUtils.encryptPassword(password));
            userService.insertUser(sysUser);
        } else {
            sysUser.setPassword(SecurityUtils.encryptPassword(password));
            sysUser.setUpdateBy("AppId登录更新用户密码");
            userService.updateUser(sysUser);
        }
        return loginService.unifiedLogin(appId, password);
    }

    /**
     * 推送,修改班组信息
     * 限流规则[30秒内最多请求100次，限流策略IP]
     * @param req 请求信息
     * @author JiangYuQi
     * @date 2024-01-13
     */
    @ApiOperation(value = "推送班组信息")
    @RateLimiter(time = 30, limitType = LimitType.IP)
    @PostMapping("/v1/pushLabourGroup")
    public AjaxResult pushLabourGroup(@Validated @RequestBody LabourSignetVo req) {
        SysApplyConfig sysApplyConfig = redisCache.getCacheObject(CacheConstants.YANZHU_SYSTEM_CONFIG+super.getUsername());
        if(req.checkTimestamp()){
            try {
                if(StringUtils.equals(ShiFouEnum.SHI.getCode()+"",sysApplyConfig.getIsDel())){
                    throw new ServiceException(HttpStatusEnum.DISABLE.getInfo(),HttpStatusEnum.DISABLE.getCode());
                }
                String result = AuthRsaUtils.decryptByPrivateKey(sysApplyConfig.getPrivateKey(),req.getSign());
                SurProjectAttendanceGroup surProjectAttendanceGroup =  JSONObject.parseObject(result, SurProjectAttendanceGroup.class);
                if(StringUtils.isNotEmpty(surProjectAttendanceGroup.getServerid())){
                    // 查询当前班组是否已推送
                    SurProjectAttendanceGroup searchModel = new SurProjectAttendanceGroup();
                    searchModel.setAppId(sysApplyConfig.getAppId());
                    searchModel.setServerid(surProjectAttendanceGroup.getServerid());
                    List<SurProjectAttendanceGroup> list = surProjectAttendanceGroupService.selectSurProjectAttendanceGroupList(searchModel);
                    if(CollectionUtils.isNotEmpty(list)){
                        surProjectAttendanceGroup.setId(list.get(0).getId());
                        surProjectAttendanceGroup.setCreateBy(list.get(0).getCreateBy());
                        surProjectAttendanceGroup.setCreateTime(list.get(0).getCreateTime());
                        surProjectAttendanceGroup.setUpdateBy(sysApplyConfig.getAppId());
                        surProjectAttendanceGroup.setUpdateTime(new Date());
                        surProjectAttendanceGroupService.updateSurProjectAttendanceGroup(surProjectAttendanceGroup);
                    }else{
                        surProjectAttendanceGroup.setCreateBy(sysApplyConfig.getAppId());
                        surProjectAttendanceGroup.setCreateTime(new Date());
                        surProjectAttendanceGroupService.insertSurProjectAttendanceGroup(surProjectAttendanceGroup);
                    }
                }else{
                    throw new ServiceException(HttpStatusEnum.DARA_EXCEPTION.getInfo(),HttpStatusEnum.DARA_EXCEPTION.getCode());
                }
            }catch (Exception e){
                throw new ServiceException(HttpStatusEnum.SINGET_EXCEPTION.getInfo(),HttpStatusEnum.SINGET_EXCEPTION.getCode());
            }
        }else{
            throw new ServiceException(HttpStatusEnum.SINGET_TIMEOUT.getInfo(),HttpStatusEnum.SINGET_TIMEOUT.getCode());
        }
        return success();
    }

    /**
     * 批量推送,修改班组列表信息
     * 限流规则[30秒内最多请求10次，限流策略IP]
     * @param req 请求信息
     * @author JiangYuQi
     * @date 2024-01-13
     */
    @ApiOperation(value = "推送班组列表信息")
    @RateLimiter(time = 30, count = 10, limitType = LimitType.IP)
    @PostMapping("/v1/pushLabourGroupList")
    public AjaxResult pushLabourGroupList(@Validated @RequestBody LabourSignetVo req) {
        // 失败集合
        List<Integer> failServiceIdList = new ArrayList<>();
        SysApplyConfig sysApplyConfig = redisCache.getCacheObject(CacheConstants.YANZHU_SYSTEM_CONFIG+super.getUsername());
        if(req.checkTimestamp()){
            try {
                if(StringUtils.equals(ShiFouEnum.SHI.getCode()+"",sysApplyConfig.getIsDel())){
                    throw new ServiceException(HttpStatusEnum.DISABLE.getInfo(),HttpStatusEnum.DISABLE.getCode());
                }
                // 保存集合
                List<SurProjectAttendanceGroup> saveList = new ArrayList<>();
                String result = AuthRsaUtils.decryptByPrivateKey(sysApplyConfig.getPrivateKey(),req.getSign());
                List<SurProjectAttendanceGroup> surProjectAttendanceGroupList =  JSON.parseArray(result,SurProjectAttendanceGroup.class);
                if(CollectionUtils.isNotEmpty(surProjectAttendanceGroupList)){
                    // 批量删除已保存数据
                    List<String> params = new ArrayList<>();
                    for(SurProjectAttendanceGroup surProjectAttendanceGroup:surProjectAttendanceGroupList){
                        params.add(sysApplyConfig.getAppId()+"-"+surProjectAttendanceGroup.getServerid());
                    }
                    surProjectAttendanceGroupService.deleteSurProjectAttendanceGroupByParams(params);
                    for(int i=0;i<surProjectAttendanceGroupList.size();i++){
                        SurProjectAttendanceGroup surProjectAttendanceGroup = surProjectAttendanceGroupList.get(i);
                        if(StringUtils.isNotEmpty(surProjectAttendanceGroup.getServerid())){
                            surProjectAttendanceGroup.setCreateBy(sysApplyConfig.getAppId());
                            surProjectAttendanceGroup.setCreateTime(new Date());
                            saveList.add(surProjectAttendanceGroup);
                        }else{
                            failServiceIdList.add(i);
                        }
                    }
                    surProjectAttendanceGroupService.batchSurProjectAttendanceGroup(saveList);
                }else{
                    throw new ServiceException(HttpStatusEnum.DARA_EXCEPTION.getInfo(),HttpStatusEnum.DARA_EXCEPTION.getCode());
                }
            }catch (Exception e){
                throw new ServiceException(HttpStatusEnum.SINGET_EXCEPTION.getInfo(),HttpStatusEnum.SINGET_EXCEPTION.getCode());
            }
        }else{
            throw new ServiceException(HttpStatusEnum.SINGET_TIMEOUT.getInfo(),HttpStatusEnum.SINGET_TIMEOUT.getCode());
        }
        return success(failServiceIdList);
    }

    /**
     * 推送,修改人员信息
     * 限流规则[30秒内最多请求100次，限流策略IP]
     * @param req 请求信息
     * @author JiangYuQi
     * @date 2024-01-13
     */
    @ApiOperation(value = "推送人员信息")
    @RateLimiter(time = 30, limitType = LimitType.IP)
    @PostMapping("/v1/pushLabourUser")
    public AjaxResult pushLabourUser(@Validated @RequestBody LabourSignetVo req) {
        SysApplyConfig sysApplyConfig = redisCache.getCacheObject(CacheConstants.YANZHU_SYSTEM_CONFIG+super.getUsername());
        if(req.checkTimestamp()){
            try {
                if(StringUtils.equals(ShiFouEnum.SHI.getCode()+"",sysApplyConfig.getIsDel())){
                    throw new ServiceException(HttpStatusEnum.DISABLE.getInfo(),HttpStatusEnum.DISABLE.getCode());
                }
                String result = AuthRsaUtils.decryptByPrivateKey(sysApplyConfig.getPrivateKey(),req.getSign());
                SurProjectAttendanceUser surProjectAttendanceUser =  JSONObject.parseObject(result, SurProjectAttendanceUser.class);
                if(StringUtils.isNotEmpty(surProjectAttendanceUser.getWorkerId())){
                    // 查询当前人员是否已推送
                    SurProjectAttendanceUser searchModel = new SurProjectAttendanceUser();
                    searchModel.setAppId(sysApplyConfig.getAppId());
                    searchModel.setWorkerId(surProjectAttendanceUser.getWorkerId());
                    List<SurProjectAttendanceUser> list = surProjectAttendanceUserService.selectSurProjectAttendanceUserList(searchModel);
                    if(CollectionUtils.isNotEmpty(list)){
                        surProjectAttendanceUser.setId(list.get(0).getId());
                        surProjectAttendanceUser.setCreateBy(list.get(0).getCreateBy());
                        surProjectAttendanceUser.setCreateTime(list.get(0).getCreateTime());
                        surProjectAttendanceUser.setUpdateBy(sysApplyConfig.getAppId());
                        surProjectAttendanceUser.setUpdateTime(new Date());
                        surProjectAttendanceUserService.updateSurProjectAttendanceUser(surProjectAttendanceUser);
                    }else{
                        surProjectAttendanceUser.setCreateBy(sysApplyConfig.getAppId());
                        surProjectAttendanceUser.setCreateTime(new Date());
                        surProjectAttendanceUserService.insertSurProjectAttendanceUser(surProjectAttendanceUser);
                    }
                }else{
                    throw new ServiceException(HttpStatusEnum.DARA_EXCEPTION.getInfo(),HttpStatusEnum.DARA_EXCEPTION.getCode());
                }
            }catch (Exception e){
                throw new ServiceException(HttpStatusEnum.SINGET_EXCEPTION.getInfo(),HttpStatusEnum.SINGET_EXCEPTION.getCode());
            }
        }else{
            throw new ServiceException(HttpStatusEnum.SINGET_TIMEOUT.getInfo(),HttpStatusEnum.SINGET_TIMEOUT.getCode());
        }
        return success();
    }

    /**
     * 批量推送,修改人员信息
     * 限流规则[30秒内最多请求10次，限流策略IP]
     * @param req 请求信息
     * @author JiangYuQi
     * @date 2024-01-13
     */
    @ApiOperation(value = "推送人员信息")
    @RateLimiter(time = 30, count = 10, limitType = LimitType.IP)
    @PostMapping("/v1/pushLabourUserList")
    public AjaxResult pushLabourUserList(@Validated @RequestBody LabourSignetVo req) {
        // 失败集合
        List<Integer> failServiceIdList = new ArrayList<>();
        SysApplyConfig sysApplyConfig = redisCache.getCacheObject(CacheConstants.YANZHU_SYSTEM_CONFIG+super.getUsername());
        if(req.checkTimestamp()){
            try {
                if(StringUtils.equals(ShiFouEnum.SHI.getCode()+"",sysApplyConfig.getIsDel())){
                    throw new ServiceException(HttpStatusEnum.DISABLE.getInfo(),HttpStatusEnum.DISABLE.getCode());
                }
                // 保存集合
                List<SurProjectAttendanceUser> saveList = new ArrayList<>();
                String result = AuthRsaUtils.decryptByPrivateKey(sysApplyConfig.getPrivateKey(),req.getSign());
                List<SurProjectAttendanceUser> surProjectAttendanceUserList =  JSON.parseArray(result,SurProjectAttendanceUser.class);
                if(CollectionUtils.isNotEmpty(surProjectAttendanceUserList)){
                    // 批量删除已保存数据
                    List<String> params = new ArrayList<>();
                    for(SurProjectAttendanceUser surProjectAttendanceUser:surProjectAttendanceUserList){
                        params.add(sysApplyConfig.getAppId()+"-"+surProjectAttendanceUser.getWorkerId());
                    }
                    surProjectAttendanceUserService.deleteSurProjectAttendanceUserByParams(params);
                    for(int i=0;i<surProjectAttendanceUserList.size();i++){
                        SurProjectAttendanceUser surProjectAttendanceUser = surProjectAttendanceUserList.get(i);
                        if(StringUtils.isNotEmpty(surProjectAttendanceUser.getWorkerId())){
                            surProjectAttendanceUser.setCreateBy(sysApplyConfig.getAppId());
                            surProjectAttendanceUser.setCreateTime(new Date());
                            saveList.add(surProjectAttendanceUser);
                        }else{
                            failServiceIdList.add(i);
                        }
                    }
                    surProjectAttendanceUserService.batchSurProjectAttendanceUser(saveList);
                }else{
                    throw new ServiceException(HttpStatusEnum.DARA_EXCEPTION.getInfo(),HttpStatusEnum.DARA_EXCEPTION.getCode());
                }
            }catch (Exception e){
                throw new ServiceException(HttpStatusEnum.SINGET_EXCEPTION.getInfo(),HttpStatusEnum.SINGET_EXCEPTION.getCode());
            }
        }else{
            throw new ServiceException(HttpStatusEnum.SINGET_TIMEOUT.getInfo(),HttpStatusEnum.SINGET_TIMEOUT.getCode());
        }
        return success(failServiceIdList);
    }

    /**
     * 推送,修改班组人员考勤信息
     * 限流规则[30秒内最多请求255次，限流策略IP]
     * @param req 请求信息
     * @author JiangYuQi
     * @date 2024-01-13
     */
    @ApiOperation(value = "推送班组人员考勤信息")
    @RateLimiter(time = 30, count = 255, limitType = LimitType.IP)
    @PostMapping("/v1/pushLabourData")
    public AjaxResult pushLabourData(@Validated @RequestBody LabourSignetVo req) {
        SysApplyConfig sysApplyConfig = redisCache.getCacheObject(CacheConstants.YANZHU_SYSTEM_CONFIG+super.getUsername());
        if(req.checkTimestamp()){
            try {
                if(StringUtils.equals(ShiFouEnum.SHI.getCode()+"",sysApplyConfig.getIsDel())){
                    throw new ServiceException(HttpStatusEnum.DISABLE.getInfo(),HttpStatusEnum.DISABLE.getCode());
                }
                String result = AuthRsaUtils.decryptByPrivateKey(sysApplyConfig.getPrivateKey(),req.getSign());
                SurProjectAttendanceData surProjectAttendanceData =  JSONObject.parseObject(result, SurProjectAttendanceData.class);
                if(StringUtils.isNotEmpty(surProjectAttendanceData.getWorkerId()) && StringUtils.isNotEmpty(surProjectAttendanceData.getServerid())){
                    // 查询当前班组人员考勤是否已推送
                    SurProjectAttendanceData searchModel = new SurProjectAttendanceData();
                    searchModel.setAppId(sysApplyConfig.getAppId());
                    searchModel.setWorkerId(surProjectAttendanceData.getWorkerId());
                    searchModel.setWorkerId(surProjectAttendanceData.getServerid());
                    List<SurProjectAttendanceData> list = surProjectAttendanceDataService.selectSurProjectAttendanceDataList(searchModel);
                    if(CollectionUtils.isNotEmpty(list)){
                        surProjectAttendanceData.setId(list.get(0).getId());
                        surProjectAttendanceData.setCreateBy(list.get(0).getCreateBy());
                        surProjectAttendanceData.setCreateTime(list.get(0).getCreateTime());
                        surProjectAttendanceData.setUpdateBy(sysApplyConfig.getAppId());
                        surProjectAttendanceData.setUpdateTime(new Date());
                        surProjectAttendanceDataService.updateSurProjectAttendanceData(surProjectAttendanceData);
                    }else{
                        surProjectAttendanceData.setCreateBy(sysApplyConfig.getAppId());
                        surProjectAttendanceData.setCreateTime(new Date());
                        surProjectAttendanceDataService.insertSurProjectAttendanceData(surProjectAttendanceData);
                    }
                }else{
                    throw new ServiceException(HttpStatusEnum.DARA_EXCEPTION.getInfo(),HttpStatusEnum.DARA_EXCEPTION.getCode());
                }
            }catch (Exception e){
                throw new ServiceException(HttpStatusEnum.SINGET_EXCEPTION.getInfo(),HttpStatusEnum.SINGET_EXCEPTION.getCode());
            }
        }else{
            throw new ServiceException(HttpStatusEnum.SINGET_TIMEOUT.getInfo(),HttpStatusEnum.SINGET_TIMEOUT.getCode());
        }
        return success();
    }

    /**
     * 批量推送,修改班组人员考勤信息
     * 限流规则[30秒内最多请求10次，限流策略IP]
     * @param req 请求信息
     * @author JiangYuQi
     * @date 2024-01-13
     */
    @ApiOperation(value = "推送班组人员考勤信息")
    @RateLimiter(time = 30, count = 10, limitType = LimitType.IP)
    @PostMapping("/v1/pushLabourDataList")
    public AjaxResult pushLabourDataList(@Validated @RequestBody LabourSignetVo req) {
        // 失败集合
        List<Integer> failServiceIdList = new ArrayList<>();
        SysApplyConfig sysApplyConfig = redisCache.getCacheObject(CacheConstants.YANZHU_SYSTEM_CONFIG+super.getUsername());
        if(req.checkTimestamp()){
            try {
                if(StringUtils.equals(ShiFouEnum.SHI.getCode()+"",sysApplyConfig.getIsDel())){
                    throw new ServiceException(HttpStatusEnum.DISABLE.getInfo(),HttpStatusEnum.DISABLE.getCode());
                }
                // 保存集合
                List<SurProjectAttendanceData> saveList = new ArrayList<>();
                String result = AuthRsaUtils.decryptByPrivateKey(sysApplyConfig.getPrivateKey(),req.getSign());
                List<SurProjectAttendanceData> surProjectAttendanceDataList =  JSON.parseArray(result,SurProjectAttendanceData.class);
                if(CollectionUtils.isNotEmpty(surProjectAttendanceDataList)){
                    // 批量删除已保存数据
                    List<String> params = new ArrayList<>();
                    for(SurProjectAttendanceData surProjectAttendanceData:surProjectAttendanceDataList){
                        params.add(sysApplyConfig.getAppId()+"-"+surProjectAttendanceData.getServerid()+"-"+surProjectAttendanceData.getWorkerId());
                    }
                    surProjectAttendanceDataService.deleteSurProjectAttendanceDataByParams(params);
                    for(int i=0;i<surProjectAttendanceDataList.size();i++){
                        SurProjectAttendanceData surProjectAttendanceData = surProjectAttendanceDataList.get(i);
                        if(StringUtils.isNotEmpty(surProjectAttendanceData.getWorkerId()) && StringUtils.isNotEmpty(surProjectAttendanceData.getServerid())){
                            surProjectAttendanceData.setCreateBy(sysApplyConfig.getAppId());
                            surProjectAttendanceData.setCreateTime(new Date());
                            saveList.add(surProjectAttendanceData);
                        }else{
                            failServiceIdList.add(i);
                        }
                    }
                    surProjectAttendanceDataService.batchSurProjectAttendanceData(saveList);
                }else{
                    throw new ServiceException(HttpStatusEnum.DARA_EXCEPTION.getInfo(),HttpStatusEnum.DARA_EXCEPTION.getCode());
                }
            }catch (Exception e){
                throw new ServiceException(HttpStatusEnum.SINGET_EXCEPTION.getInfo(),HttpStatusEnum.SINGET_EXCEPTION.getCode());
            }
        }else{
            throw new ServiceException(HttpStatusEnum.SINGET_TIMEOUT.getInfo(),HttpStatusEnum.SINGET_TIMEOUT.getCode());
        }
        return success(failServiceIdList);
    }

}
